//
//  CATransform3D+Extras.h
//  Procreate
//
//  Created by Lloyd Bottomley on 26/06/2014.
//  Copyright (c) 2014 Savage Interactive Pty Ltd. All rights reserved.
//

@import QuartzCore;

static inline CGFloat CATransform3DRotationOnZ(CATransform3D transform);
static inline CGFloat CATransform3DScaleOnZ(CATransform3D transform);
static inline CGFloat CATransform3DScaleOfX(CATransform3D transform);
static inline CGFloat CATransform3DScaleOfY(CATransform3D transform);
static inline CGPoint CATransform3DTranslateOrigin(CATransform3D transform);
static inline BOOL CATransform3DIsFlippedX(CATransform3D transform);
static inline BOOL CATransform3DIsFlippedY(CATransform3D transform);

static inline CGFloat CATransform3DRotationOnZ(CATransform3D transform)
{
	CGFloat angle = (CGFloat)atan2(transform.m12, transform.m11);
	return angle;
}
static inline CGFloat CATransform3DScaleOnZ(CATransform3D transform)
{
	CGFloat x = CATransform3DScaleOfX(transform);
	CGFloat y = CATransform3DScaleOfY(transform);
	return (x + y) * 0.5f;
}
static inline CGFloat CATransform3DScaleOfX(CATransform3D transform)
{
	CGFloat a = transform.m11;
	CGFloat c = transform.m21;
	return (CGFloat)sqrt(a * a + c * c);
}
static inline CGFloat CATransform3DScaleOfY(CATransform3D transform)
{
	CGFloat a = transform.m12;
	CGFloat c = transform.m22;
	return (CGFloat)sqrt(a * a + c * c);
}
static inline CGPoint CATransform3DTranslateOrigin(CATransform3D transform)
{
	return CGPointApplyAffineTransform(CGPointZero, CATransform3DGetAffineTransform(CATransform3DInvert(transform)));
}

static inline BOOL CATransform3DIsFlippedX(CATransform3D transform)
{
	return transform.m11 < 0 && transform.m22 > 0;
}
static inline BOOL CATransform3DIsFlippedY(CATransform3D transform)
{
	return transform.m11 > 0 && transform.m22 < 0;
}
