//
//  CGFloat+Extras.h
//  Procreate
//
//  Created by Lloyd Bottomley on 26/06/2014.
//  Copyright (c) 2014 Savage Interactive Pty Ltd. All rights reserved.
//

@import CoreGraphics;

#define SiPI (CGFloat)M_PI
#define SiPI2 (CGFloat)M_PI_2
#define SiPI4 (CGFloat)M_PI_4
#define SiPI6 (CGFloat)(M_PI / 6.0)

#define SiDegreesToRadians(a) ((a) / 57.29578f)
#define SiRadiansToDegrees(a) ((a) * 57.29578f)

#define SiFloatIsEqual(a, b, z) (a == b || fabs(a - b) < z)

#define SiCosineCurve(timeValue) ((1 - (CGFloat)cos(timeValue * SiPI)) * 0.5f)
#define SiSineHalfCurve(timeValue) ((CGFloat)sin(timeValue * SiPI2))



static inline CGFloat CGFloatMIN(CGFloat a, CGFloat b)
{
	return a < b ? a : b;
}
static inline CGFloat CGFloatMAX(CGFloat a, CGFloat b)
{
	return a > b ? a : b;
}
static inline CGFloat CGFloatCLAMP(CGFloat value, CGFloat min, CGFloat max)
{
	return CGFloatMIN(CGFloatMAX(value, min), max);
}

static inline CGFloat CGFloatLERP(CGFloat min, CGFloat max, CGFloat z)
{
	return min + ((max - min) * z);
}

static inline BOOL CGFloatIsWithinRangeOfOffset(CGFloat x, CGFloat offset)
{
	const CGFloat min = offset - SiPI4;
	const CGFloat max = min + SiPI2;

	return x >= min && x <= max;
}

static inline BOOL CGFloatSnapValue(CGFloat *value, CGFloat division, CGFloat distance)
{
	CGFloat remainder = (CGFloat)fmod(fabs(*value), division);
	BOOL snap = remainder < distance || remainder > (division - distance);
	if (snap) {
		*value = (CGFloat)round(*value / division) * division;
	}

	return snap;
}

static inline CGFloat CGFloatStretchValue(CGFloat value, CGFloat start)
{
	BOOL negative = value < 0;
	if (negative) {
		value = -value;
	}

	CGFloat max = negative ? 1 : start;
	if (!max) {
		return 0;
	}

	value = value <= max ? value : max * (CGFloat)sqrt(((value - max) / max) + 1.0f);
	value = (CGFloat)round(value);

	return negative ? -value : value;
}


#import "CGFloat+Division.h"
