//
//  CGPoint+Extras.h
//  Procreate
//
//  Created by Lloyd Bottomley on 26/06/2014.
//  Copyright (c) 2014 Savage Interactive Pty Ltd. All rights reserved.
//

@import CoreGraphics;

static inline CGPoint CGPointMinus(CGPoint a, CGPoint b)
{
	return CGPointMake(a.x - b.x, a.y - b.y);
}
static inline CGPoint CGPointAdd(CGPoint a, CGPoint b)
{
	return CGPointMake(a.x + b.x, a.y + b.y);
}
static inline CGPoint CGPointMultiply(CGPoint a, CGPoint b)
{
	return CGPointMake(a.x * b.x, a.y * b.y);
}
static inline CGPoint CGPointDivide(CGPoint a, CGPoint b)
{
	return CGPointMake(a.x / b.x, a.y / b.y);
}

static inline CGFloat CGPointDistance(CGPoint a)
{
	return (CGFloat)sqrt(a.x * a.x + a.y * a.y);
}
static inline CGFloat CGPointDistanceBetween(CGPoint a, CGPoint b)
{
	return CGPointDistance(CGPointMinus(a, b));
}

static inline CGFloat CGPointAngle(CGPoint a)
{
	return (CGFloat)atan2(a.y, a.x);
}
static inline CGFloat CGPointAngleBetween(CGPoint a, CGPoint b)
{
	return CGPointAngle(CGPointMinus(a, b));
}

static inline CGPoint CGPointFromAngle(CGFloat a, CGFloat l)
{
	return CGPointMake((CGFloat)cos(a) * l, (CGFloat)sin(a) * l);
}
static inline CGPoint CGPointFromCGSize(CGSize a)
{
	return CGPointMake(a.width, a.height);
}

static inline CGPoint CGPointLERP(CGPoint a, CGPoint b, CGFloat c)
{
	return CGPointMake(CGFloatLERP(a.x, b.x, c), CGFloatLERP(a.y, b.y, c));
}
static inline CGPoint CGPointCLAMP(CGPoint a, CGPoint b, CGPoint c)
{
	return CGPointMake(CGFloatCLAMP(a.x, b.x, c.x), CGFloatCLAMP(a.y, b.y, c.y));
}

static inline CGPoint CGPointRound(CGPoint a)
{
	return CGPointMake((CGFloat)round(a.x), (CGFloat)round(a.y));
}
static inline CGPoint CGPointCenter(CGPoint a, CGPoint b)
{
	return CGPointMake((a.x + b.x) * 0.5f, (a.y + b.y) * 0.5f);
}

static inline BOOL CGPointEqualToPointWithTolerance(CGPoint a, CGPoint b, CGFloat tolerance)
{
	return SiFloatIsEqual(a.x, b.x, tolerance) && SiFloatIsEqual(a.y, b.y, tolerance);
}

static inline BOOL CGPointSnapPoint(CGPoint *point, CGFloat division, CGFloat distance, CGPoint origin)
{
	CGFloat length = CGPointDistance((*point));
	CGFloat angle = 0;

	while (angle <= (SiPI * 2))
	{
		CGPoint snap = CGPointFromAngle(angle, length);
		CGPoint amount = CGPointMinus(snap, *point);
		if (CGPointDistance(amount) < distance)
		{
			*point = snap;
			return YES;
		}

		angle += division;
	}

	return NO;
}

#import "CGPoint+Dot.h"
