//
//  SiColor.h
//  Procreate
//
//  Created by Lloyd Bottomley on 4/02/13.
//  Copyright (c) 2013 Savage Interactive Pty Ltd. All rights reserved.
//

@import Foundation;

#import "../Si+Essentials.h"


#if SI_TARGET_IOS
@import UIKit;
#else
@import AppKit;
#endif

typedef double SiColorFloat;

static inline SiColorFloat SiColorFloatMIN(SiColorFloat a, SiColorFloat b)
{
	return a < b ? a : b;
}
static inline SiColorFloat SiColorFloatMAX(SiColorFloat a, SiColorFloat b)
{
	return a > b ? a : b;
}
static inline SiColorFloat SiColorFloatCLAMP(SiColorFloat value, SiColorFloat min, SiColorFloat max)
{
	return SiColorFloatMIN(SiColorFloatMAX(value, min), max);
}

@interface SiColor : NSObject <NSCoding>

@property (nonatomic, readonly) SiColorFloat hue;
@property (nonatomic, readonly) SiColorFloat saturation;
@property (nonatomic, readonly) SiColorFloat brightness;
@property (nonatomic, readonly) SiColorFloat alpha;


+ (SiColor *)clearColor;

+ (SiColor *)colorWithHue:(SiColorFloat)hue saturation:(SiColorFloat)saturation brightness:(SiColorFloat)brightness alpha:(SiColorFloat)alpha;
+ (SiColor *)colorWithHex:(NSUInteger)hex;
+ (SiColor *)colorWithRed:(SiColorFloat)red green:(SiColorFloat)green blue:(SiColorFloat)blue;
+ (SiColor *)colorWithDictionary:(NSDictionary *)dictionary;

- (NSUInteger)hexValue;
- (CGColorRef)CGColorCopy CF_RETURNS_RETAINED;

#if SI_TARGET_IOS
- (UIColor *)UIColorValue;
#endif

- (NSDictionary *)toDictionary;

@end


#if SI_TARGET_IOS
@interface UIColor (SiColor)

- (SiColor *)SiColorValue;

@end

#define UIColorFromRGB(x) [[SiColor colorWithHex:x] UIColorValue]
#endif
