//
//  SiPSD.h
//
//  Created by Lloyd Bottomley on 12/04/11.
//  Copyright 2011 Savage Interactive Pty Ltd. All rights reserved.
//

@import Foundation;
@import CoreGraphics;

@class SiPSD;


typedef enum {
	kSiPSDBlendNormal,
	kSiPSDBlendMultiply,
	kSiPSDBlendScreen,
	kSiPSDBlendAdd,
	kSiPSDBlendLighten,
	kSiPSDBlendExclusion,

	kSiPSDBlendDifference,
	kSiPSDBlendSubtract,
	kSiPSDBlendLinearBurn,
	kSiPSDBlendColorDodge,
	kSiPSDBlendColorBurn,
	kSiPSDBlendOverlay,
	kSiPSDBlendHardLight,
	kSiPSDBlendColor,
	kSiPSDBlendLuminosity,
	kSiPSDBlendHue,
	kSiPSDBlendSaturation,
	kSiPSDBlendSoftLight,
} kSiPSDBlend;


@protocol SiPSDDataSource

- (CGSize)documentSizeOfPSD:(SiPSD *)aPSD;
- (CGFloat)documentDPIOfPSD:(SiPSD *)aPSD;
- (NSData *)documentCompositeForPSD:(SiPSD *)aPSD;
- (NSUInteger)numberOfLayersInPSD:(SiPSD *)aPSD;

- (NSData *)psd:(SiPSD *)aPSD layerDataAtIndex:(NSUInteger)layerIndex;
- (CGRect)psd:(SiPSD *)aPSD layerRectAtIndex:(NSUInteger)layerIndex;
- (NSString *)psd:(SiPSD *)aPSD layerNameAtIndex:(NSUInteger)layerIndex;
- (CGFloat)psd:(SiPSD *)aPSD layerOpacityAtIndex:(NSUInteger)layerIndex;
- (BOOL)psd:(SiPSD *)aPSD layerVisibleAtIndex:(NSUInteger)layerIndex;
- (BOOL)psd:(SiPSD *)aPSD layerPreservedAtIndex:(NSUInteger)layerIndex;
- (kSiPSDBlend)psd:(SiPSD *)aPSD layerBlendAtIndex:(NSUInteger)layerIndex;

@end

@protocol SiPSDDelegate

@optional

- (void)psd:(SiPSD *)aPSD completedLayerAtIndex:(NSUInteger)layerIndex;
- (void)completedExportOfPSD:(SiPSD *)aPSD;
- (BOOL)respondsToSelector:(SEL)aSelector;

@end


@interface SiPSD : NSObject

@property (nonatomic, readonly) BOOL cancelled;
@property (nonatomic, weak) id <SiPSDDataSource> dataSource;
@property (nonatomic, weak) id <SiPSDDelegate> delegate;

- (void)writeToPath:(NSString *)path;
- (void)completedLayerAtIndex:(NSUInteger)layerIndex;

- (void)cancel;

@end
