//
//  SiDialogController.h
//
//  Created by Lloyd Bottomley on 28/05/2014.
//  Copyright (c) 2014 Savage Interactive. All rights reserved.
//

#import "../../Si+Essentials.h"


#import "SiOrientedViewController.h"
#import "SiSubContainingController.h"
#import "UIViewController+Dialog.h"

@class SiDialogController;

extern const CGFloat SiDialogControllerRepositionDuration;

@protocol SiDialogControllerDelegate <NSObject>

@optional
- (CGPoint)dialogViewControllerAnchorPoint:(SiDialogController *)dialogViewController inPresentationView:(UIView *)presentationView;

- (CGRect)dialogViewController:(SiDialogController *)dialogViewController availableDisplayAreaWithPresentationArea:(CGRect)presentationArea inPresentationView:(UIView *)presentationView;
- (CGRect)dialogViewController:(SiDialogController *)dialogViewController adjustedDisplayArea:(CGRect)displayArea inPresentationView:(UIView *)presentationView;

- (BOOL)shouldPresentDialogViewController:(SiDialogController *)dialogViewController;
- (void)didPresentDialogViewController:(SiDialogController *)dialogViewController;

- (BOOL)shouldDismissDialogViewController:(SiDialogController *)dialogViewController;
- (void)didDismissDialogViewController:(SiDialogController *)dialogViewController;

@end


@interface SiDialogController : SiOrientedViewController <SiContainingControllerProtocol>

@property (nonatomic, strong, readonly) UIViewController *rootViewController;
@property (nonatomic, weak) id <SiDialogControllerDelegate> delegate;
@property (nonatomic, strong) NSArray *passthroughViews;
@property (nonatomic, strong) NSArray *passthroughGestures;


- (id)initWithRootViewController:(UIViewController *)rootViewController;

- (void)presentAnimated:(BOOL)animated;
- (void)presentAnimated:(BOOL)animated fromView:(UIView *)fromView edges:(SiEdge)edges;

- (void)dismissAnimated:(BOOL)animated;
- (void)dismissAnimated:(BOOL)animated completionBlock:(void (^)(void))completionBlock;

- (void)hideAnimated:(BOOL)animated;
- (void)repositionAnimated:(BOOL)animated;
- (void)showAnimated:(BOOL)animated;

@end
