//
//  SiTableView+Private.h
//  Procreate
//
//  Created by Lloyd Bottomley on 7/12/11.
//  Copyright (c) 2011 Savage Interactive. All rights reserved.
//

#import "SiTableView.h"
#import "SiTableViewCell+Private.h"
#import "SiTableViewEntity.h"


#define SiTableViewAnimationDuration 0.3f
#define SiTableViewScrollMaxDistance 80
#define SiTableViewScrollStartSpeed 2.0f
#define SiTableViewScrollStartAcceleration 0.06f


typedef enum {
	SiTableViewDirectionUp,
	SiTableViewDirectionDown,
} SiTableViewDirection;


static NSString *const SiTableViewTransactionToIndex = @"SiTableViewTransactionToIndex";
static NSString *const SiTableViewTransactionObject = @"SiTableViewTransactionObject";
static NSString *const SiTableViewTransactionType = @"SiTableViewTransactionType";

enum {
	SiTableViewTransactionTypeUpdate,
	SiTableViewTransactionTypeResize,
	SiTableViewTransactionTypeSelect,
	SiTableViewTransactionTypeDeselect,
	SiTableViewTransactionTypeBeginEditing,
	SiTableViewTransactionTypeStopEditing,
	SiTableViewTransactionTypeExpand,
	SiTableViewTransactionTypeShrink,
};



@interface SiTableView ()

@property (strong, nonatomic) UIView *contentView;
@property (nonatomic) CGSize previousSize;
@property (nonatomic) CGFloat contentMaxWidth;
@property (nonatomic) CGFloat contentMaxHeight;

#pragma mark Actions
@property (strong, nonatomic) NSMutableSet *contexts;
@property (strong, nonatomic) NSMutableSet *insertingEntities;
@property (strong, nonatomic) NSMutableArray *insertingIndices;
@property (strong, nonatomic) NSMutableArray *deletingIndices;
@property (strong, nonatomic) NSMutableArray *updateTransactions;
@property (strong, nonatomic) NSMutableArray *movingTransactions;
@property (nonatomic) BOOL updating;

#pragma mark Dragging
@property (strong, nonatomic) NSMutableArray *draggingEntities;
@property (nonatomic) CGPoint draggingCenter;
@property (nonatomic) BOOL dragging;
@property (nonatomic) NSUInteger hoveringIndex;
@property (nonatomic) BOOL hoveringExpand;
@property (nonatomic) CGPoint hoveringPoint;

#pragma mark Entities
@property (nonatomic) NSUInteger firstVisibleIndex;
@property (nonatomic) NSUInteger lastVisibleIndex;

@property (strong, nonatomic) NSMutableSet *selectedEntities;
@property (strong, nonatomic) NSMutableSet *editingEntities;
@property (strong, nonatomic) NSMutableSet *expandedEntities;

@property (nonatomic) BOOL layoutRequired;
@property (nonatomic) BOOL insertedHorizontally;

@property (strong, nonatomic) NSMutableSet *visibleCells;
@property (strong, nonatomic) NSMutableSet *deletingCells;
@property (strong, nonatomic) NSMutableSet *movingCells;
@property (strong, nonatomic) NSMutableDictionary *queuedCells;
@property (strong, nonatomic) NSMutableArray *entities;

#pragma mark Scrolling
@property (weak, nonatomic) CADisplayLink *scrollTimer;
@property (nonatomic) CGFloat scrollSpeed;
@property (nonatomic) CGFloat scrollAcceleration;
@property (strong, nonatomic) NSMutableSet *touchesBegan;


- (void)initSiTableView;

- (void)destroyTable;
- (void)createTable;

- (void)assertCount;
- (void)updateTableSize;

@end



#pragma mark -
#pragma mark Actions

@interface SiTableView (PrivateActions)

- (void)animateRowsDown;

- (void)commitUpdatesInsertBatch;
- (void)commitUpdatesDeleteBatch;
- (void)commitUpdatesMoveBatch;
- (void)commitUpdatesTransactionBatch;

- (void)commitUpdatesBeginLayoutWithDuration:(CGFloat)duration completionBlock:(void (^)(BOOL finished))completionBlock;

@end



#pragma mark -
#pragma mark Dragging

@interface SiTableView (PrivateDragging)

- (void)putInCellsCommitAnimated:(BOOL)animated;

- (void)hoveringAtPageScrollUpFinished;
- (void)hoveringAtPageScrollDownFinished;
- (void)hoveringBeganAtPoint:(CGPoint)point expand:(BOOL)expand;
- (void)hoveringStopped;

- (void)showHoverBarAtIndex:(NSUInteger)anIndex;
- (void)hideHoverBar;

- (void)separateCellsAtIndex:(NSUInteger)anIndex;
- (void)uniteSeparatedCells;
- (void)animateSeparation;

@end



#pragma mark -
#pragma mark Entities

@interface SiTableView (PrivateEntities)

- (SiTableViewEntity *)entityAtPoint:(CGPoint)point;

- (void)setupEntities;
- (void)reflowEntities;

- (void)updateVisibleIndexes;
- (void)reclaimInvisibleEntities;
- (void)layoutVisibleEntities;

- (SiTableViewEntity *)createEntityForIndex:(NSUInteger)anIndex;
- (SiTableViewEntity *)updateEntityAtIndex:(NSUInteger)anIndex;

- (BOOL)isPoint:(CGPoint)point afterEntity:(SiTableViewEntity *)entity halved:(BOOL)halved;

- (SiTableViewCell *)setupCellForEntity:(SiTableViewEntity *)entity placeholder:(BOOL)isPlaceholder;
- (SiTableViewCell *)cellForEntity:(SiTableViewEntity *)entity;
- (SiTableViewCell *)placeholderForEntity:(SiTableViewEntity *)entity;

@end



#pragma mark -
#pragma mark Scrolling

@interface SiTableView (PrivateScrolling)

- (void)cancelCellTouchesBegan;

- (void)startScrollDirection:(SiTableViewDirection)direction;
- (void)stopScroll;
- (void)scrollInterval;

@end
