//
//  SiDragManager.h
//  Sorted
//
//  Created by Lloyd Bottomley on 31/07/11.
//  Copyright 2011 Savage Interactive Pty Ltd. All rights reserved.
//

@import UIKit;

@class SiDragManager;


typedef enum {
	SiDragManagerTransitionNone,
	SiDragManagerTransitionDisolve,
} SiDragManagerTransition;

typedef enum {
	SiDragManagerReceivedNotAccepted,
	SiDragManagerReceivedAccepted,
	SiDragManagerReceivedConsumed,
} SiDragManagerReceived;

@protocol SiDragManagerReceiver <NSObject>

- (NSSet *)dragManager:(SiDragManager *)manager receivableViewsForKey:(NSString *)key;
- (BOOL)dragManager:(SiDragManager *)manager canAcceptDrop:(NSString *)key inView:(UIView *)aView atPoint:(CGPoint)point userInfo:(NSDictionary *)userInfo;
- (void)dragManager:(SiDragManager *)manager receiveDrop:(NSString *)key inView:(UIView *)aView atPoint:(CGPoint)point userInfo:(NSDictionary *)userInfo;

@optional
- (void)dragManager:(SiDragManager *)manager dropWillCommit:(NSString *)key received:(SiDragManagerReceived)received userInfo:(NSDictionary *)userInfo;
- (void)dragManager:(SiDragManager *)manager dropDidCommit:(NSString *)key received:(SiDragManagerReceived)received userInfo:(NSDictionary *)userInfo;

- (CGPoint)dragManager:(SiDragManager *)manager offsetForKey:(NSString *)key inView:(UIView *)aView atPoint:(CGPoint)point userInfo:(NSDictionary *)userInfo;

@end


extern NSString *const SiDragManagerWillBeginDragKey;
extern NSString *const SiDragManagerDidFinishDragKey;


@interface SiDragManager : NSObject

+ (SiDragManager *)sharedManager;

- (void)registerForDrop:(id <SiDragManagerReceiver>)observer name:(NSString *)key;
- (void)unregisterForDrop:(id <SiDragManagerReceiver>)observer name:(NSString *)key;

- (void)beginDraggingView:(UIView *)aView gesture:(UIGestureRecognizer *)gesture forKey:(NSString *)key userInfo:(NSDictionary *)userInfo;
- (void)beginDraggingViews:(NSSet *)views gesture:(UIGestureRecognizer *)gesture forKey:(NSString *)key userInfo:(NSDictionary *)userInfo;
- (void)dragMoved:(UIGestureRecognizer *)gesture forKey:(NSString *)key;
- (void)dragEnded:(UIGestureRecognizer *)gesture forKey:(NSString *)key;
- (void)dragCancelledForKey:(NSString *)key;

@end
