//
//  SiDraggableCollectionViewDataSource.h
//  Procreate
//
//  Created by Dafydd Williams on 23/06/2014.
//  Copyright (c) 2014 Savage Interactive Pty Ltd. All rights reserved.
//

@import UIKit;


// REFACTOR: none of this is data-source, mostly it's view-delegate stuff
@protocol SiDraggableCollectionViewDataSource <UICollectionViewDataSource>

@required

// DISCUSS:
// The @required interface here seems to largely duplicate the @optional one.
//
// The canVerbItemAtIndexPath and didVerbItemAtIndexPath seem to follow the
// established pattern of view data sources and delegates, but this seems to be
// the view 'commanding' the model to update, without implying when or why.
//
// IMO, the view should ask the model if a view-level operation may be performed
// ( `canVerb`), then inform it that it did (`didVerb`); as implied by the
// @optional interface below. What does this add? -- Rowan 2014-11-10

// DOCUMENT: when is this called relative to `collectionView:didMoveItemAtIndexPath:toIndexPath:`?
- (void)collectionView:(UICollectionView *)collectionView moveItemAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath;

// REFACTOR: not optional, like collectionViewCanMoveItems? is this called anyway, then optionally overriden by that?
// DOCUMENT: how does it differ from `collectionView:canMoveItemAtIndexPath:toIndexPath`?
- (BOOL)collectionView:(UICollectionView *)collectionView canMoveItemAtIndexPath:(NSIndexPath *)indexPath;

// DOCUMENT: how does this differ from `collectionView:didDeleteItemAtIndexPath:`
- (void)collectionView:(UICollectionView *)collectionView deleteItemAtIndexPath:(NSIndexPath *)indexPath;

// REFACTOR: data source should not be controlling appearance, should be appearanceDelegate.
- (CGFloat)uniformScaleForDragInCollectionView:(UICollectionView *)collectionView;
- (UIImage *)shadowForDragInCollectionView:(UICollectionView *)collectionView;
- (UIImage *)backviewImageForDragInCollectionView:(UICollectionView *)collectionView;


@optional

- (BOOL)collectionViewCanMoveItems:(UICollectionView *)collectionView;
- (BOOL)collectionView:(UICollectionView *)collectionView canMoveItemAtIndexPath:(NSIndexPath *)indexPath toIndexPath:(NSIndexPath *)toIndexPath;
- (void)collectionView:(UICollectionView *)collectionView didMoveItemAtIndexPath:(NSIndexPath *)indexPath toIndexPath:(NSIndexPath *)toIndexPath;
- (BOOL)collectionView:(UICollectionView *)collectionView canDeleteItemAtIndexPath:(NSIndexPath *)indexPath;
- (void)collectionView:(UICollectionView *)collectionView didDeleteItemAtIndexPath:(NSIndexPath *)indexPath;

@end
