uniform mediump float uHue;
uniform mediump float uRadius;

varying mediump vec2 discPos;

mediump vec3 hsv2rgb(mediump vec3 c)
{
    mediump vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    mediump vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

mediump float topo(mediump float value)
{
	mediump float m = mod(value, 0.1);
	return float(m > 0.095) + float(m < 0.005);
}

mediump float squareEase(mediump vec2 position)
{
	const mediump float sqrt2 = sqrt(2.0);

	mediump float funcX = position.x / sqrt2;
	mediump float heightOfTopAbovePoint = 0.95 - funcX * funcX;

	mediump float result = clamp(mix(0.5, 1.0, position.y / heightOfTopAbovePoint), 0.0, 1.0);

	return result;
}

mediump vec4 squareBlend(mediump vec2 position)
{
	const mediump vec2 midSB = vec2(0.5, 0.5);

	mediump float saturation = squareEase(position.yx);
	mediump float brightness = squareEase(position * vec2(-1.0, 1.0));

	// return vec4(saturation, topo(saturation), topo(brightness), 1.0);
	return vec4(hsv2rgb(vec3(uHue, saturation, brightness)), 1.0);
}

void main()
{
	mediump vec4 color = squareBlend(discPos);
	mediump float len = length(discPos);
	mediump float pixel = 1.0 / uRadius;
	mediump float alpha = 1.0 - clamp((len - 1.0 + pixel) * uRadius, 0.0, 1.0);
	color = color * alpha;
	gl_FragColor = color;
}
