//
//  Silica.h
//  Procreate
//
//  Created by Lloyd Bottomley on 20/12/10.
//  Copyright 2010 Savage Interactive Pty Ltd. All rights reserved.
//

@import Foundation;
@import QuartzCore;


#import "Silica+Types.h"
#import "SilicaGL+Types.h"

#import "SilicaView.h"
#import "SilicaDocument.h"

#import "SilicaBrush.h"
#import "SilicaBrushOptions.h"

#import "SilicaShape.h"

#import "SilicaFilter.h"
#import "SilicaFilterOpacity.h"
#import "SilicaDataSource.h"


extern NSString *const SilicaAnimatingDidFinishNotification;

extern NSString *const SilicaSuccessKey;
extern NSString *const SilicaProgressKey;
extern NSString *const SilicaLayerKey;


@class SilicaComposite;

typedef void (^SilicaCanvasCallbackBlock)(SilicaComposite *composite, CGSize size);


@interface Silica : NSObject

@property (nonatomic, strong, readonly) SilicaDocument *document;
@property (nonatomic, readonly) BOOL animating;
@property (nonatomic, readonly) BOOL rendering;
@property (nonatomic, readonly) BOOL loaded;
@property (nonatomic, readonly) BOOL realtime;

@property (nonatomic) CATransform3D canvas;
@property (nonatomic) BOOL canvasBlur;
@property (nonatomic) GLColor128 canvasBlurColor;
@property (nonatomic, readonly) CATransform3D canvasFlipped;
@property (nonatomic, strong) SilicaCanvasCallbackBlock canvasCallback;

@property (nonatomic, strong) SilicaShape *shape;

@property (nonatomic) BOOL mask;
@property (nonatomic) GLfloat maskAlpha;
@property (nonatomic) BOOL maskVisible;
@property (nonatomic, readonly) CGRect maskRect;

@property (nonatomic) BOOL strokeCombining;
@property (nonatomic) BOOL transformCopy;

@property (nonatomic) BOOL puck;
@property (nonatomic) GLColor128 puckColor;
@property (nonatomic) CGPoint puckPosition;

@property (nonatomic, readonly) CGFloat strokeLength;


- (id)initWithDataSource:(NSObject <SilicaDataSource> *)dataSource;

- (void)shutdown;
- (CGImageRef)updateThumbnail CF_RETURNS_RETAINED;
- (void)updateComposite;
- (void)preloadBrushTextures;

- (CGImageRef)imageForLayer:(SilicaLayer *)aLayer rect:(CGRect)rect size:(CGSize)size orientation:(SilicaDocumentOrientation)orientation masked:(BOOL)masked CF_RETURNS_RETAINED;
- (CGImageRef)imageForCompositeOfSize:(CGSize)size orientation:(SilicaDocumentOrientation)orientation masked:(BOOL)masked CF_RETURNS_RETAINED;
- (CGImageRef)imageForCanvasOfSize:(CGSize)size CF_RETURNS_RETAINED;

- (void)animateToCanvasTransform:(CATransform3D)transform duration:(CGFloat)duration callbackBlock:(void (^)(void))callbackBlock completionBlock:(void (^)(void))completionBlock;
- (void)animateCanvasFlipX:(CGFloat)x y:(CGFloat)y duration:(CGFloat)duration callbackBlock:(void (^)(void))callbackBlock completionBlock:(void (^)(void))completionBlock;

- (void)beginAnimatingViewport;
- (void)forceRepaintViewport;
- (void)finishAnimatingViewport;

- (void)beginRealtimeCompositeWithLayer:(SilicaLayer *)aLayer type:(SilicaRenderType)type;
- (void)refreshRealtimeComposite:(BOOL)realtime;
- (void)finishRealtimeComposite:(BOOL)commit;

- (void)animateLayer:(SilicaLayer *)aLayer angle:(CGFloat)angle axis:(GLVertex3D)axis anchor:(GLVertex3D)anchor duration:(CGFloat)duration callbackBlock:(void (^)(void))callbackBlock completionBlock:(void (^)(void))completionBlock;
- (void)animateLayer:(SilicaLayer *)aLayer scale:(GLVertex3D)scale anchor:(GLVertex3D)anchor duration:(CGFloat)duration callbackBlock:(void (^)(void))callbackBlock completionBlock:(void (^)(void))completionBlock;
- (void)animateLayer:(SilicaLayer *)aLayer opacity:(CGFloat)opacity duration:(CGFloat)duration callbackBlock:(void (^)(void))callbackBlock completionBlock:(void (^)(void))completionBlock;
- (void)cancelAllAnimations;

- (void)strokeBegan:(CGPoint)point;
- (void)strokeMoved:(CGPoint)point;
- (void)strokeEnded:(CGPoint)point;
- (void)strokeCanceled:(CGPoint)point;

- (CGPoint)transformPointToCanvas:(CGPoint)point;
- (CGPoint)transformPointFromCanvas:(CGPoint)point;
- (CGRect)transformRectToCanvas:(CGRect)rect;
- (CGRect)transformRectFromCanvas:(CGRect)rect;

- (void)floodWithColor:(GLColor128)aColor atPoint:(CGPoint)point threshold:(CGFloat)aThreshold sampleLayer:(SilicaLayer *)sampleLayer recomposite:(BOOL)recomposite floodRectCallback:(void (^)(CGRect))floodRectCallback;
- (void)commitFloodAndContinue;

- (void)clearLayer:(SilicaLayer *)aLayer withColor:(GLColor128)aColor;

- (void)loadMaskFromLayer:(SilicaLayer *)aLayer;
- (void)loadMaskFromShape:(SilicaShape *)shape mode:(SilicaMaskMode)mode;
- (void)invertMask;

- (GLColor128)colorAtPoint:(CGPoint)point;
- (CGPoint)clampedPoint:(CGPoint)point;

@end
