//
//  SilicaBrush.h
//  Procreate
//
//  Created by Lloyd Bottomley on 29/10/12.
//  Copyright (c) 2012 Savage Interactive Pty Ltd. All rights reserved.
//

#import "SilicaGL+Types.h"
#import "SilicaDocument+Types.h"
#import "SilicaBrush+Types.h"


#define SilicaBrushThumbWidth 440
#define SilicaBrushThumbHeight 276


extern NSString *const SilicaBrushShapeChangedNotificationKey;
extern NSString *const SilicaBrushGrainChangedNotificationKey;


@interface SilicaBrush : NSObject

@property (nonatomic) GLfloat paintSize;
@property (nonatomic) GLfloat paintOpacity;
@property (nonatomic) GLfloat smudgeSize;
@property (nonatomic) GLfloat smudgeOpacity;
@property (nonatomic) GLfloat eraseSize;
@property (nonatomic) GLfloat eraseOpacity;
@property (nonatomic) GLColor128 color;

@property (nonatomic) BOOL shapeRandomise;
@property (nonatomic) GLfloat shapeRotation;
@property (nonatomic) GLfloat shapeScatter;
@property (nonatomic) BOOL shapeInverted;
@property (nonatomic) SilicaDocumentOrientation shapeOrientation;

@property (nonatomic) GLfloat textureMovement;
@property (nonatomic) GLfloat textureScale;
@property (nonatomic) GLfloat textureZoom;
@property (nonatomic) BOOL textureFilter;
@property (nonatomic) GLfloat textureRotation;
@property (nonatomic) BOOL textureInverted;
@property (nonatomic) SilicaDocumentOrientation textureOrientation;

@property (nonatomic) GLfloat plotSpacing;
@property (nonatomic) GLfloat plotJitter;

@property (nonatomic) GLfloat taperStartLength;
@property (nonatomic) GLfloat taperEndLength;
@property (nonatomic) GLfloat taperOpacity;
@property (nonatomic) GLfloat taperSize;

@property (nonatomic) GLfloat dynamicsSpeedOpacity;
@property (nonatomic) GLfloat dynamicsSpeedSize;
@property (nonatomic) GLfloat dynamicsPressureOpacity;
@property (nonatomic) GLfloat dynamicsPressureSize;
@property (nonatomic) GLfloat dynamicsJitterOpacity;
@property (nonatomic) GLfloat dynamicsJitterSize;
@property (nonatomic) GLfloat dynamicsMix;
@property (nonatomic) GLfloat dynamicsFalloff;
@property (nonatomic) GLfloat dynamicsLoad;
@property (nonatomic) BOOL dynamicsGlaze;

@property (nonatomic) GLfloat maxOpacity;
@property (nonatomic) GLfloat minOpacity;
@property (nonatomic) GLfloat maxSize;
@property (nonatomic) GLfloat minSize;

@property (nonatomic, strong, readonly) NSString *displayName;
@property (nonatomic, strong) NSString *name;
@property (nonatomic) BOOL oriented;
@property (nonatomic) BOOL stamp;

@property (nonatomic, strong) NSString *bundledShapePath;
@property (nonatomic, strong) NSString *bundledGrainPath;


- (GLfloat)sizeForTool:(SilicaBrushTool)tool;
- (GLfloat)opacityForTool:(SilicaBrushTool)tool;
- (void)setSize:(GLfloat)size forTool:(SilicaBrushTool)tool;
- (void)setOpacity:(GLfloat)opacity forTool:(SilicaBrushTool)tool;

- (CGImageRef)grainImage CF_RETURNS_RETAINED;
- (CGImageRef)shapeImage CF_RETURNS_RETAINED;
- (void)setGrainImage:(CGImageRef)cgImage;
- (void)setShapeImage:(CGImageRef)cgImage;

- (GLfloat)maximumMaxSize;

+ (void)extractBrushNamed:(NSString *)name toPath:(NSString *)aPath;
+ (void)loadBrushAtPath:(NSString *)aPath isPreset:(BOOL)isPreset withCompletionBlock:(void (^)(SilicaBrush *aBrush))completionBlock wait:(BOOL)wait;
- (void)saveBrushWithCompletionBlock:(void (^)(void))completionBlock wait:(BOOL)wait;

- (void)loadThumbnailInverted:(BOOL)inverted withCompletionBlock:(void (^)(CGImageRef anImage))completionBlock;
- (void)invalidateThumbnail;

+ (SilicaBrush *)duplicateBrush:(SilicaBrush *)aBrush toPath:(NSString *)aPath;

+ (CGImageRef)loadBrushImageWithImageSource:(CGImageSourceRef)imageSource inverted:(BOOL)inverted CF_RETURNS_RETAINED;
+ (CGImageRef)loadBrushImageFromBundledFile:(NSString *)aPath inverted:(BOOL)inverted CF_RETURNS_RETAINED;
+ (CGImageRef)loadBrushImageFromFile:(NSString *)aPath inverted:(BOOL)inverted CF_RETURNS_RETAINED;

@end
