//
//  SilicaComposite.h
//  Procreate
//
//  Created by Lloyd Bottomley on 21/12/10.
//  Copyright 2010 Savage Interactive Pty Ltd. All rights reserved.
//


#import "Silica.h"
#import "SilicaCompositeDelegate.h"


@class SilicaView, SilicaPackage, SilicaState, SilicaBuffer;


@interface SilicaComposite : NSObject

@property (nonatomic, weak) id <SilicaCompositeDelegate> delegate;

@property (nonatomic, strong) SilicaView *view;
@property (nonatomic, readonly) BOOL animating;

@property (nonatomic, strong) SilicaShape *shape;

@property (nonatomic) BOOL mask;
@property (nonatomic) GLfloat maskAlpha;
@property (nonatomic) BOOL puck;
@property (nonatomic) GLColor128 puckColor;
@property (nonatomic) CGPoint puckPosition;
@property (nonatomic) GLfloat puckAngle;

@property (nonatomic) BOOL canvasBlur;
@property (nonatomic) GLColor128 canvasBlurColor;
@property (nonatomic, strong, readonly) SilicaBuffer *canvasBlurBuffer;
@property (nonatomic, strong) SilicaCanvasCallbackBlock canvasCallback;


- (id)initWithState:(SilicaState *)aState;

- (void)compositeTiles:(SilicaPackage *)dirtyTiles;
- (void)compositeTiles:(SilicaPackage *)dirtyTiles withSelectedBuffer:(SilicaBuffer *)selectedBuffer realtimeScale:(BOOL)realtimeScale;

- (void)forceUpdateCanvas;
- (void)paintCanvasToBuffer:(SilicaBuffer *)aBuffer;

- (void)animateToCanvasTransform:(CATransform3D)transform duration:(CGFloat)duration callbackBlock:(void (^)(void))callbackBlock completionBlock:(void (^)(void))completionBlock;
- (void)animateCanvasRotation:(CGSize)aRotation duration:(CGFloat)duration callbackBlock:(void (^)(void))callbackBlock completionBlock:(void (^)(void))completionBlock;
- (CATransform3D)animatedCanvasTransform;

- (void)compositeLayers:(NSArray *)layers toBuffer:(SilicaBuffer *)aBuffer;

- (void)beginAnimating;
- (void)finishAnimating;

@end
