//
//  SilicaDocument.h
//  Procreate
//
//  Created by Lloyd Bottomley on 19/06/12.
//  Copyright (c) 2012 Savage Interactive Pty Ltd. All rights reserved.
//

#import "SilicaDocument+Types.h"
#import "Silica+Types.h"

#import "SilicaLayer.h"

@import Si;


@class VideoManager;
@class VideoSegmentInfo;

extern NSString *const SilicaDocumentLayerKey;
extern NSString *const SilicaDocumentLayerIndexKey;
extern NSString *const SilicaDocumentLayerToIndexKey;
extern NSString *const SilicaDocumentLayersMergedKey;
extern NSString *const SilicaDocumentChangeTypeKey;

extern NSString *const SilicaDocumentDictionaryPathKey;
extern NSString *const SilicaDocumentDictionaryNameKey;
extern NSString *const SilicaDocumentDictionarySizeKey;
extern NSString *const SilicaDocumentDictionaryOrientationKey;
extern NSString *const SilicaDocumentDictionaryFlippedHorizonallyKey;
extern NSString *const SilicaDocumentDictionaryFlippedVerticallyKey;
extern NSString *const SilicaDocumentDictionaryBackgroundColorKey;
extern NSString *const SilicaDocumentDictionaryBackgroundHiddenKey;
extern NSString *const SilicaDocumentDictionaryCompositeKey;
extern NSString *const SilicaDocumentDictionaryLayersKey;
extern NSString *const SilicaDocumentDictionarySelectedLayerIndexKey;
extern NSString *const SilicaDocumentDictionarySelectedSamplerLayerIndexKey;

extern NSString *const SilicaDocumentDidChangeLayersNotification;
extern NSString *const SilicaDocumentDidSaveNotification;


@interface SilicaDocument : NSObject <SiPSDDataSource>

@property (nonatomic, readonly) BOOL needsExpansion;
@property (nonatomic, strong) NSString *name;
@property (nonatomic, strong, readonly) NSString *displayName;
@property (nonatomic, strong, readonly) NSString *path;
@property (nonatomic, readonly) CGSize size;

@property (nonatomic, readonly) unsigned long long fileSize;

@property (nonatomic) GLColor128 backgroundColor;
@property (nonatomic) BOOL backgroundHidden;

@property (nonatomic) SilicaDocumentOrientation orientation;
@property (nonatomic) BOOL flippedHorizontally;
@property (nonatomic) BOOL flippedVertically;

@property (nonatomic, strong, readonly) SiUndoManager *undoManager;

@property (nonatomic, strong, readonly) VideoManager *videoManager;
@property (nonatomic, strong, readonly) VideoSegmentInfo *videoSegmentInfo;
@property (nonatomic, assign, readonly) BOOL videoPurged;

@property (nonatomic, readonly) NSTimeInterval trackedCreativeTime;
@property (nonatomic, assign) BOOL trackingTime;

- (id)initWithPath:(NSString *)aPath size:(CGSize)aSize;
- (id)initWithDictionary:(NSDictionary *)dictionary;

+ (CGFloat)angleFromOrientation:(SilicaDocumentOrientation)anOrientation;
+ (SilicaDocumentOrientation)orientationFromAngle:(CGFloat)angle;

+ (SilicaDocument *)duplicateDocument:(SilicaDocument *)aDocument toPath:(NSString *)aPath withProgressBlock:(void (^)(CGFloat progress))progressBlock completionBlock:(void (^)(SilicaDocument *duplicateDocument))completionBlock wait:(BOOL)wait;

- (NSArray *)allLayers;
- (NSUInteger)layerCount;
- (SilicaLayer *)layerAtIndex:(NSUInteger)anIndex;

- (SilicaLayer *)selectedLayer;
- (NSUInteger)selectedLayerIndex;
- (SilicaLayer *)selectedSamplerLayer;
- (NSUInteger)selectedSamplerLayerIndex;

- (SilicaLayer *)composite;
- (SilicaLayer *)mask;

- (SilicaLayer *)selectLayerAtIndex:(NSUInteger)anIndex;
- (SilicaLayer *)selectSamplerLayerAtIndex:(NSUInteger)anIndex;
- (SilicaLayer *)insertLayerAtIndex:(NSUInteger)anIndex image:(CGImageRef)anImage orientation:(SilicaDocumentOrientation)orientation withCompletionBlock:(void (^)(SilicaLayer *aLayer))completionBlock;
- (SilicaLayer *)deleteLayerAtIndex:(NSUInteger)anIndex;

- (SilicaLayer *)moveLayerAtIndex:(NSUInteger)anIndex toIndex:(NSUInteger)toIndex;
- (SilicaLayer *)duplicateLayerAtIndex:(NSUInteger)anIndex toIndex:(NSUInteger)toIndex withCompletionBlock:(void (^)(SilicaLayer *aLayer))completionBlock;
- (SilicaLayer *)mergeFromLayerAtIndex:(NSUInteger)anIndex toIndex:(NSUInteger)toIndex withCompletionBlock:(void (^)(SilicaLayer *aLayer))completionBlock;

+ (void)loadDocumentAtPath:(NSString *)aPath withCompletionBlock:(void (^)(SilicaDocument *aDocument))completionBlock wait:(BOOL)wait;
- (void)saveDocumentWithCompletionBlock:(void (^)(void))completionBlock wait:(BOOL)wait;

- (void)loadLayersWithProgressBlock:(void (^)(CGFloat progress))progressBlock completionBlock:(void (^)(BOOL success))completionBlock;
- (void)unloadLayersAndWait:(BOOL)wait;
- (void)eraseUndoDirectoryContents;

- (BOOL)thumbnailIsSaving;
- (CGSize)thumbnailSize;
- (NSString *)thumbnailResolvedPath;
- (void)loadThumbnailWithCompletionBlock:(void (^)(CGImageRef anImage))completionBlock;

- (void)loadVideoRecorder;
- (void)unloadVideoRecorder;
- (void)savedVideoSegment:(VideoSegmentInfo *)segmentInfo;
- (void)purgeVideo;

- (void)deleteEntireDocument:(BOOL)really;

@end
