/*
 *  SilicaGL+Types.h
 *  Procreate
 *
 *  Created by Lloyd Bottomley on 8/01/11.
 *  Copyright 2011 Savage Interactive Pty Ltd. All rights reserved.
 *
 */

@import QuartzCore;
@import CoreGraphics;
@import CoreVideo;
@import OpenGLES.ES2;
@import ImageIO;
@import Si;

#define GL_PI (GLfloat)M_PI
#define GL_PI2 (GLfloat)M_PI_2

// 2D vertex
typedef struct {
	GLfloat x;
	GLfloat y;
} GLVertex2D;

// 3D vertex
typedef struct {
	GLfloat x;
	GLfloat y;
	GLfloat z;
} GLVertex3D;

// 4D vertex
typedef struct {
	GLfloat x;
	GLfloat y;
	GLfloat z;
	GLfloat w;
} GLVertex4D;

// 2D Quad
typedef struct {
	GLVertex2D d0;
	GLVertex2D p0;
	GLVertex2D p1;
	GLVertex2D p2;
	GLVertex2D p3;
	GLVertex2D d3;
} GLQuad2D;

// 2D Quad
typedef struct {
	GLushort d0;
	GLushort i0;
	GLushort i1;
	GLushort i2;
	GLushort i3;
	GLushort d3;
} GLQuadIndex;

// 2D Quad
typedef struct {
	GLfloat d0;
	GLfloat f0;
	GLfloat f1;
	GLfloat f2;
	GLfloat f3;
	GLfloat d3;
} GLQuadFloat;

// 2x2 matrix
typedef struct {
	GLfloat m11;
	GLfloat m12;
	GLfloat m21;
	GLfloat m22;
} GLMatrix2x2;

// 3x3 matrix
typedef struct {
	GLfloat m11;
	GLfloat m12;
	GLfloat m13;
	GLfloat m21;
	GLfloat m22;
	GLfloat m23;
	GLfloat m31;
	GLfloat m32;
	GLfloat m33;
} GLMatrix3x3;

// 4x4 matrix
typedef struct {
	GLfloat m11;
	GLfloat m12;
	GLfloat m13;
	GLfloat m14;
	GLfloat m21;
	GLfloat m22;
	GLfloat m23;
	GLfloat m24;
	GLfloat m31;
	GLfloat m32;
	GLfloat m33;
	GLfloat m34;
	GLfloat m41;
	GLfloat m42;
	GLfloat m43;
	GLfloat m44;
} GLMatrix4x4;

// RGBA 32bit
typedef struct {
	GLubyte r;
	GLubyte g;
	GLubyte b;
	GLubyte a;
} GLColor32;

// RGBA 128bit
typedef struct {
	GLfloat r;
	GLfloat g;
	GLfloat b;
	GLfloat a;
} GLColor128;

// Color Quad
typedef struct {
	GLColor128 d0;
	GLColor128 p0;
	GLColor128 p1;
	GLColor128 p2;
	GLColor128 p3;
	GLColor128 d3;
} GLQuadColor128;


typedef enum {
	SilicaFormatByte,
	SilicaFormatHalfFloat,
	SilicaFormatFloat,
} SilicaFormat;

typedef enum {
	SilicaMaskModeReplace,
	SilicaMaskModeAdd,
	SilicaMaskModeSubtract,
} SilicaMaskMode;



#if DEBUGGL
#define SilicaDebugGLCheck() do {	\
GLenum SilicaDebugGLCheckError = glGetError();	\
ZAssert(SilicaDebugGLCheckError == GL_NO_ERROR, @"Debugging GL encountered error: %x", SilicaDebugGLCheckError);	\
} while (0);
#else
#define SilicaDebugGLCheck() do {} while (0);
#endif


#define GLColor128From32(color32) (GLColor128){ color32.r / 255.0f, color32.g / 255.0f, color32.b / 255.0f, color32.a / 255.0f }
#define GLColor32From128(color128) (GLColor32){ (GLubyte)(color128.r * 255), (GLubyte)(color128.g * 255), (GLubyte)(color128.b * 255), (GLubyte)(color128.a * 255) }

#define GLColor128Clear (GLColor128){ 0, 0, 0, 0 }
#define GLColor128Black (GLColor128){ 0, 0, 0, 1 }
#define GLColor128White (GLColor128){ 1, 1, 1, 1 }

#define GLColor32Clear (GLColor32){ 0, 0, 0, 0 }
#define GLColor32Black (GLColor32){ 0, 0, 0, 255 }
#define GLColor32White (GLColor32){ 255, 255, 255, 255 }


static inline BOOL GLColor32NotEqualsGLColor32(GLColor32 colorA, GLColor32 colorB) {
	return (colorA.a != colorB.a || colorA.r != colorB.r || colorA.g != colorB.g || colorA.b != colorB.b);
}
static inline BOOL GLColor32EqualsGLColor32(GLColor32 colorA, GLColor32 colorB) {
	return (colorA.r == colorB.r && colorA.g == colorB.g && colorA.b == colorB.b && colorA.a == colorB.a);
}
static inline BOOL GLColor128EqualsGLColor128(GLColor128 colorA, GLColor128 colorB) {
	return (colorA.r == colorB.r && colorA.g == colorB.g && colorA.b == colorB.b && colorA.a == colorB.a);
}

static inline GLColor128 GLColor128Demultiplied(GLColor128 color) {
	return color.a == 0 ? GLColor128Clear : (GLColor128){ color.r / color.a, color.g / color.a, color.b / color.a, color.a };
}
static inline GLColor128 GLColor128Multiply(GLColor128 color) {
	return (GLColor128){ color.r * color.a, color.g * color.a, color.b * color.a, color.a };
}

#define GLColor32DemultiplyComponent(c, a) ((255 * c + (a / 2)) / a)
static inline GLColor32 GLColor32Demultiplied(GLColor32 color) {
	return color.a == 0 ? GLColor32Clear : (GLColor32){ GLColor32DemultiplyComponent(color.r, color.a), GLColor32DemultiplyComponent(color.g, color.a), GLColor32DemultiplyComponent(color.b, color.a), color.a };
}
#define GLColor32MultiplyComponent(c, a) ((c * a + 127) / 255)
static inline GLColor32 GLColor32Multiplied(GLColor32 color) {
	return (GLColor32){ GLColor32MultiplyComponent(color.r, color.a), GLColor32MultiplyComponent(color.g, color.a), GLColor32MultiplyComponent(color.b, color.a), color.a };
}

GLMatrix3x3 GLMatrix3x3FromCGAffineTransform(CGAffineTransform transform);
GLMatrix4x4 GLMatrix4x4FromCATransform3D(CATransform3D transform);

BOOL GLMatrix4x4EqualToMatrix(GLMatrix4x4 a, GLMatrix4x4 b);
BOOL GLMatrix4x4IsTranslation(GLMatrix4x4 matrix);
GLfloat GLMatrix4x4RotationOnZ(GLMatrix4x4 transform);
GLfloat GLMatrix4x4ScaleOnZ(GLMatrix4x4 transform);
GLfloat GLMatrix4x4ScaleOfX(GLMatrix4x4 transform);
GLfloat GLMatrix4x4ScaleOfY(GLMatrix4x4 transform);

GLVertex4D GLVertex4DApplyMatrix4x4(GLMatrix4x4 matrix, GLVertex4D input);
GLVertex2D GLVertex2DApplyMatrix4x4(GLMatrix4x4 matrix, GLVertex2D input);

CGFloat GLVertex2DAngleBetween(GLVertex2D a, GLVertex2D b);
CGFloat GLVertex2DDistanceBetween(GLVertex2D a, GLVertex2D b);
CGFloat GLVertex2DDistanceBetweenPointAndLine(GLVertex2D point, GLVertex2D lineStart, GLVertex2D lineEnd, GLVertex2D *intersection);
GLVertex2D GLVertex2DLinesIntersection(GLVertex2D line1Start, GLVertex2D line1End, GLVertex2D line2Start, GLVertex2D line2End);
GLVertex2D GLVertex2DIntersectionVectorForLineAndPoint(GLVertex2D lineStart, GLVertex2D lineEnd, GLVertex2D point, GLVertex2D vectorStart);
BOOL GLVertex2DAreLinesIntersecting(GLVertex2D line1Start, GLVertex2D line1End, GLVertex2D line2Start, GLVertex2D line2End, GLVertex2D *intersection);

GLVertex2D GLVertex2DMakeFromDirection(CGFloat angle, CGFloat distance);

static inline GLVertex2D GLVertex2DMake(GLfloat x, GLfloat y)
{
	return (GLVertex2D) { x, y };
}
static inline GLVertex3D GLVertex3DMake(GLfloat x, GLfloat y, GLfloat z)
{
	return (GLVertex3D) { x, y, z };
}

CGFloat GLVertex3DDistanceBetween(GLVertex3D point1, GLVertex3D point2);
CGFloat GLVertex3DDistanceBetweenPointAndLine(GLVertex3D point, GLVertex3D lineStart, GLVertex3D lineEnd, GLVertex3D *intersection);

CGRect CGRectContainingGLQuad2D(GLQuad2D quad);

GLQuad2D GLQuad2DMake(GLVertex2D topLeft, GLVertex2D topRight, GLVertex2D bottomLeft, GLVertex2D bottomRight);
GLQuadFloat GLQuadFloatMake(GLfloat topLeft, GLfloat topRight, GLfloat bottomLeft, GLfloat bottomRight);
GLQuad2D GLQuad2DApplyMatrix4x4(GLMatrix4x4 matrix, GLQuad2D quad);

#define GLVertex2DZero (GLVertex2D){ 0, 0 }

static inline GLfloat GLfloatMIN(GLfloat a, GLfloat b)
{
	return a < b ? a : b;
}
static inline GLfloat GLfloatMAX(GLfloat a, GLfloat b)
{
	return a > b ? a : b;
}
static inline GLfloat GLfloatCLAMP(GLfloat value, GLfloat min, GLfloat max)
{
	return GLfloatMIN(GLfloatMAX(value, min), max);
}
static inline GLfloat GLfloatLERP(GLfloat min, GLfloat max, GLfloat amount) {
	return min + ((max - min) * amount);
}
static inline GLVertex2D GLVertex2DFromCGPoint(CGPoint a) {
	return GLVertex2DMake((GLfloat)a.x, (GLfloat)a.y);
}
static inline GLVertex2D GLVertex2DFromCGSize(CGSize a) {
	return GLVertex2DMake((GLfloat)a.width, (GLfloat)a.height);
}
static inline CGPoint CGPointFromGLVertex2D(GLVertex2D a) {
	return CGPointMake(a.x, a.y);
}

static inline BOOL GLVertex2DIsEqualToVertex(GLVertex2D a, GLVertex2D b) {
	return a.x == b.x && a.y == b.y;
}
static inline GLVertex2D GLVertex2DAdd(GLVertex2D a, GLVertex2D b) {
	return GLVertex2DMake(a.x + b.x, a.y + b.y);
}
static inline GLVertex2D GLVertex2DMinus(GLVertex2D a, GLVertex2D b) {
	return GLVertex2DMake(a.x - b.x, a.y - b.y);
}
static inline GLVertex2D GLVertex2DMultiply(GLVertex2D a, GLVertex2D b) {
	return GLVertex2DMake(a.x * b.x, a.y * b.y);
}
static inline GLVertex2D GLVertex2DCenter(GLVertex2D a, GLVertex2D b) {
	return GLVertex2DMake((a.x + b.x) * 0.5f, (a.y + b.y) * 0.5f);
}
static inline GLVertex2D GLVertex2DLERP(GLVertex2D a, GLVertex2D b, GLfloat c) {
	return GLVertex2DMake(GLfloatLERP(a.x, b.x, c), GLfloatLERP(a.y, b.y, c));
}
static inline GLfloat GLVertex2DAngle(GLVertex2D a)
{
	return atan2f(a.y, a.x);
}


#define GLVertex3DZero GLVertex3DMake(0, 0, 0)
#define GLVertex3DAdd(a, b) GLVertex3DMake(a.x + b.x, a.y + b.y, a.z + b.z)
#define GLVertex3DMinus(a, b) GLVertex3DMake(a.x - b.x, a.y - b.y, a.z - b.z)

static inline BOOL GLVertex3DIsEqualToVertex(GLVertex3D a, GLVertex3D b) {
	return a.x == b.x && a.y == b.y && a.z == b.z;
}

GLQuad2D GLQuad2DFromCGRect(CGRect rect);
