//
//  SilicaRender.h
//  Procreate
//
//  Created by Lloyd Bottomley on 21/12/10.
//  Copyright 2010 Savage Interactive Pty Ltd. All rights reserved.
//
/*
 
 Rendering main
 
 */

#import "SilicaGL+Types.h"

#import "SilicaShape.h"


@class SilicaState, SilicaLayer, SilicaBuffer, SilicaComposite, SilicaBrush, SilicaStrokePlot, SilicaPackage;

@interface SilicaRender : NSObject

@property (nonatomic, strong, readonly) SilicaLayer *layer;
@property (nonatomic, readonly) SilicaRenderType type;


- (id)initWithState:(SilicaState *)aState;
- (void)unload;
- (void)delayedSetup;
- (void)preloadBrushTextures;

- (SilicaBuffer *)lockTemporaryBuffer;
- (void)unlockTemporaryBuffer;

- (CGImageRef)imageForBuffer:(SilicaBuffer *)aBuffer rect:(CGRect)rect size:(CGSize)size orientation:(SilicaDocumentOrientation)orientation incremental:(BOOL)incremental masked:(BOOL)masked CF_RETURNS_RETAINED;
- (void)loadMaskFromBuffer:(SilicaBuffer *)aBuffer;

- (void)renderInvertedMaskToBuffer:(SilicaBuffer *)aBuffer;
- (void)renderShape:(SilicaShape *)shape mode:(SilicaMaskMode)mode onBuffer:(SilicaBuffer *)aBuffer;
- (void)renderTransformForLayer:(SilicaLayer *)aLayer copy:(BOOL)copy;
- (void)renderFilterForLayer:(SilicaLayer *)aLayer;

- (void)beginRenderWithLayer:(SilicaLayer *)aLayer type:(SilicaRenderType)type;

- (SilicaPackage *)renderPlot:(SilicaStrokePlot *)plot;
- (SilicaPackage *)clearWithColor:(GLColor128)color128;
- (SilicaPackage *)floodWithColor:(GLColor128)color128 atPoint:(CGPoint)point threshold:(GLfloat)threshold sampleLayer:(SilicaLayer *)sampleLayer recompositeTiles:(SilicaPackage *)recompositeTiles floodQuad:(GLQuad2D *)floodQuad;

- (SilicaPackage *)commitRender;
- (SilicaPackage *)commitRenderAndContinue;
- (SilicaPackage *)revertRender;
- (SilicaPackage *)revertRenderAndContinue;

@end

#import "SilicaRender+Copy.h"
