//
//  SilicaShader.h
//  Procreate
//
//  Created by Lloyd Bottomley on 21/12/10.
//  Copyright 2010 Savage Interactive Pty Ltd. All rights reserved.
//
/*
 
 GL shader abstraction
 
 */


#import "SilicaGL+Types.h"


extern NSString *const uSize;
extern NSString *const uSizeSource;
extern NSString *const uSizeCombine;
extern NSString *const uSizeDestination;
extern NSString *const uSizeMask;
extern NSString *const uSizeScaled;

extern NSString *const uTextureSource;
extern NSString *const uTextureStroke;
extern NSString *const uTextureMask;
extern NSString *const uTextureSmudge;
extern NSString *const uTextureGrain;
extern NSString *const uTextureShape;
extern NSString *const uTextureCombine;
extern NSString *const uTextureOriginal;
extern NSString *const uTextureLookup;

extern NSString *const uTransform;
extern NSString *const uPerspective;
extern NSString *const uPosition;
extern NSString *const uRotation;
extern NSString *const uRadius;
extern NSString *const uDirection;
extern NSString *const uLock;

extern NSString *const uThreshold;
extern NSString *const uColor;
extern NSString *const uOuterColor;
extern NSString *const uAlpha;
extern NSString *const uCount;
extern NSString *const uTime;
extern NSString *const uWidth;
extern NSString *const uVector;
extern NSString *const uWetness;
extern NSString *const uMix;
extern NSString *const uHue;
extern NSString *const uSaturation;
extern NSString *const uBrightness;
extern NSString *const uGamma;
extern NSString *const uSharpness;
extern NSString *const uShadows;
extern NSString *const uMidtones;
extern NSString *const uHighlights;

extern NSString *const aPosition;
extern NSString *const aSide;
extern NSString *const aVector;
extern NSString *const aVectorSource;
extern NSString *const aVectorCombine;
extern NSString *const aVectorDestination;
extern NSString *const aVectorMask;

extern NSString *const aCoordGrain;
extern NSString *const aCoordShape;
extern NSString *const aCoordSmudge;

extern NSString *const aColor;
extern NSString *const aAlpha;

extern NSString *const uDissolveFromTexture;
extern NSString *const uDissolveToTexture;

@class SilicaTexture;


@interface SilicaShader : NSObject

+ (void)purge;

- (GLint)uniform:(NSString *)key;
- (GLuint)attribute:(NSString *)key;

+ (SilicaShader *)use:(NSString *)named;
- (BOOL)validate;
- (void)finish;

- (void)draw:(NSUInteger)vertices;
- (void)drawQuad2D:(NSUInteger)count;

- (void)uniform:(NSString *)key components:(GLsizei)components values:(void *)values type:(GLenum)type count:(GLsizei)count;
- (void)attribute:(NSString *)key components:(GLsizei)components values:(void *)values type:(GLenum)type stride:(GLsizei)stride length:(GLint)length;

//- (void)uniform:(NSString *)key texture:(SilicaTexture *)texture;
- (void)uniform:(NSString *)key texture:(SilicaTexture *)texture filterMag:(GLint)filterMag filterMin:(GLint)filterMin wrap:(GLint)wrap;
- (void)uniform:(NSString *)key floatValue:(GLfloat)value;
- (void)uniform:(NSString *)key integerValue:(GLint)value;
- (void)uniform:(NSString *)key boolValue:(GLboolean)value;
- (void)uniform:(NSString *)key vertex2D:(GLVertex2D)value;
- (void)uniform:(NSString *)key vertex3D:(GLVertex3D)value;
- (void)uniform:(NSString *)key vertex4D:(GLVertex4D)value;
- (void)uniform:(NSString *)key matrix2x2:(GLMatrix2x2)value;
- (void)uniform:(NSString *)key matrix3x3:(GLMatrix3x3)value;
- (void)uniform:(NSString *)key matrix4x4:(GLMatrix4x4)value;
- (void)uniform:(NSString *)key color128:(GLColor128)value;

- (void)attribute:(NSString *)key floatValues:(GLfloat *)values count:(NSUInteger)count;
- (void)attribute:(NSString *)key vertex2D:(GLVertex2D *)values count:(NSUInteger)count;
- (void)attribute:(NSString *)key vertex3D:(GLVertex3D *)values count:(NSUInteger)count;
- (void)attribute:(NSString *)key vertex4D:(GLVertex4D *)values count:(NSUInteger)count;
- (void)attribute:(NSString *)key color128:(GLColor128 *)values count:(NSUInteger)count;
- (void)attribute:(NSString *)key quad2D:(GLQuad2D *)values count:(NSUInteger)count;
- (void)attribute:(NSString *)key quadColor128:(GLQuadColor128 *)values count:(NSUInteger)count;

@end
