//
//  SilicaState.h
//  Procreate
//
//  Created by Lloyd Bottomley on 29/03/11.
//  Copyright 2011 Savage Interactive Pty Ltd. All rights reserved.
//

#import "Silica.h"

#import "SilicaStateDelegate.h"
#import "SilicaStrokeDelegate.h"


@class SiUndoManager, SilicaBrush, SilicaStroke, SilicaBrushOptions;


@interface SilicaState : NSObject <SilicaStrokeDelegate>

@property (nonatomic, weak) id <SilicaStateDelegate> delegate;
@property (nonatomic, weak) id <SilicaDataSource> dataSource;

@property (nonatomic, strong, readonly) SilicaView *view;
@property (nonatomic, strong, readonly) SilicaDocument *document;
@property (nonatomic, strong, readonly) SilicaBrush *brush;
@property (nonatomic, strong, readonly) SilicaBrushOptions *brushOptions;
@property (nonatomic, strong, readonly) SilicaStroke *stroke;

@property (nonatomic) BOOL realtime;
@property (nonatomic) BOOL mask;
@property (nonatomic) BOOL transformCopy;

@property (nonatomic) CATransform3D canvas;
@property (nonatomic, readonly) CATransform3D canvasFlipped;

@property (atomic, readonly) BOOL painting;
@property (atomic, readonly) BOOL backgrounded;

@property (nonatomic, readonly) SilicaMode mode;
@property (nonatomic, readonly) SilicaFormat format;
@property (nonatomic, readonly) CGFloat animationScale;


- (id)initWithDataSource:(id <SilicaDataSource>)aDataSource delegate:(id <SilicaStateDelegate>)aDelegate;

- (void)beginStroke:(CGPoint)point;
- (void)moveStroke:(CGPoint)point;
- (void)endStroke:(CGPoint)point;
- (void)cancelStroke:(CGPoint)point;

- (SilicaStrokePlot *)plotPoint:(CGPoint)point angle:(CGFloat)angle;

- (CGPoint)clampedPoint:(CGPoint)point;
- (GLColor32)colorAtPoint:(CGPoint)point;

- (void)compositeLayers:(NSArray *)layers toBuffer:(SilicaBuffer *)aBuffer;

- (CATransform3D)transformFlippedToDocument:(CATransform3D)transform;

@end
