
uniform highp vec2 uSizeSource;
uniform highp vec2 uSizeDestination;
uniform highp mat4 uTransform;

#if kPerspective
uniform highp mat4 uPerspective;
#endif

attribute highp vec2 aVectorSource;
attribute highp vec2 aVectorDestination;

varying highp vec4 vCoordSource;

#if kMask
attribute highp vec2 aVectorMask;
uniform highp vec2 uSizeMask;
varying highp vec4 vCoordMask;
#endif


void main()
{
#if kPerspective
	highp vec4 position = uPerspective * uTransform * vec4(aVectorDestination, 0.0, 1.0);
	position = vec4(position.xy, 1.0, position.w);
	
#else
	highp vec4 position = uTransform * vec4(aVectorDestination, 1.0, 1.0);
	position /= position.w;
#endif

	vCoordSource = vec4(aVectorSource / uSizeSource, 0.0, 1.0) * position.z;
#if kMask
	vCoordMask = vec4(aVectorMask / uSizeMask, 0.0, 1.0) * position.z;
#endif

	gl_Position = vec4(((position.xy / (uSizeDestination * 0.5)) - 1.0), 0.0, position.w);
}
