
uniform mediump sampler2D uTextureSource;
uniform mediump sampler2D uTextureMask;

uniform highp vec3 uShadows;
uniform highp vec3 uMidtones;
uniform highp vec3 uHighlights;

varying highp vec2 vCoord;
varying mediump mat2 vTransform;


mediump vec3 unpremultiply(const mediump vec4 color)
{
	return color.a == 0.0 ? vec3(0.0) : clamp(color.rgb / color.a, 0.0, 1.0);
}

mediump float rgb2y(const mediump vec3 rgb)
{
	return dot(rgb, vec3(0.299, 0.587, 0.114));
}
mediump vec2 rgb2uv(const mediump vec3 rgb)
{
	return (rgb.br - vec2(rgb2y(rgb))) * vec2(0.565, 0.713);
}

mediump vec3 yuv2rgb(const mediump vec3 yuv)
{
	mediump float r = yuv.r + (1.403 * yuv.b);
	mediump float g = yuv.r - (0.344 * yuv.g) - (0.714 * yuv.b);
	mediump float b = yuv.r + (1.770 * yuv.g);

	return vec3(r, g, b);
}

void main()
{
	mediump vec4 source = texture2D(uTextureSource, vCoord);
	mediump float mask = texture2D(uTextureMask, vCoord).r;

	mediump vec3 demultiplied = unpremultiply(source);

	const lowp float a = 0.25;
	const lowp float b = 0.333;
	const lowp float scale = 0.7;

	lowp vec3 shadows = uShadows * (clamp((demultiplied - b) / -a + 0.5, 0.0, 1.0) * scale);
	lowp vec3 midtones = uMidtones * (clamp((demultiplied - b) / a + 0.5, 0.0, 1.0) * clamp((demultiplied + b - 1.0) / -a + 0.5, 0.0, 1.0) * scale);
	lowp vec3 highlights = uHighlights * (clamp((demultiplied + b - 1.0) / a + 0.5, 0.0, 1.0) * scale);

	mediump vec3 newColor = clamp(demultiplied + shadows + midtones + highlights, 0.0, 1.0);
	mediump vec3 yuv = vec3(rgb2y(demultiplied), rgb2uv(newColor));
	gl_FragColor = mix(source, vec4(yuv2rgb(yuv) * source.a, source.a), mask);
}

