
uniform mediump sampler2D uTextureSource;
uniform mediump sampler2D uTextureMask;

uniform mediump float uBrightness;

varying highp vec2 vCoord;
varying mediump mat2 vTransform;


mediump vec3 unpremultiply(const mediump vec4 color)
{
	return color.a == 0.0 ? vec3(0.0) : clamp(color.rgb / color.a, 0.0, 1.0);
}

mediump vec3 rgb2yuv(const mediump vec3 rgb)
{
	mediump float y = dot(rgb, vec3(0.299, 0.587, 0.114));
	mediump vec2 uv = (rgb.br - vec2(y)) * vec2(0.565, 0.713);

	return vec3(y, uv);
}
mediump vec3 yuv2rgb(const mediump vec3 yuv)
{
	mediump float r = yuv.r + (1.403 * yuv.b);
	mediump float g = yuv.r - (0.344 * yuv.g) - (0.714 * yuv.b);
	mediump float b = yuv.r + (1.770 * yuv.g);

	return vec3(r, g, b);
}

void main()
{
	mediump vec4 source = texture2D(uTextureSource, vCoord);
	mediump float mask = texture2D(uTextureMask, vCoord).r;
	mediump vec3 yuv = rgb2yuv(unpremultiply(source));
	
	yuv.bg = vTransform * yuv.bg;
	yuv.r += uBrightness;

	gl_FragColor = mix(source, vec4(clamp(yuv2rgb(yuv), 0.0, 1.0) * source.a, source.a), mask);
}

