
uniform mediump float uThreshold;
uniform mediump float uMix;
uniform mediump vec4 uColor;
uniform mediump vec4 uOuterColor;

uniform mediump sampler2D uTextureSource;
uniform mediump sampler2D uTextureLookup;
uniform mediump sampler2D uTextureShape;

varying highp vec2 vCoord;
varying highp vec2 vCoordL;
varying highp vec2 vCoordR;
varying highp vec2 vCoordT;
varying highp vec2 vCoordB;
varying highp vec2 vCoordTL;
varying highp vec2 vCoordTR;
varying highp vec2 vCoordBL;
varying highp vec2 vCoordBR;

#if kMask
uniform mediump sampler2D uTextureMask;
varying highp vec2 vCoordMask;
#endif


void main()
{
	mediump vec4 texel = texture2D(uTextureSource, vCoord);
	mediump vec4 lookup = texture2D(uTextureLookup, vCoord);

	// Dilation
	mediump float lookupL = texture2D(uTextureShape, vCoordL).r;
	mediump float lookupR = texture2D(uTextureShape, vCoordR).r;
	mediump float lookupT = texture2D(uTextureShape, vCoordT).r;
	mediump float lookupB = texture2D(uTextureShape, vCoordB).r;
	mediump float lookupTL = texture2D(uTextureShape, vCoordTL).r;
	mediump float lookupTR = texture2D(uTextureShape, vCoordTR).r;
	mediump float lookupBL = texture2D(uTextureShape, vCoordBL).r;
	mediump float lookupBR = texture2D(uTextureShape, vCoordBR).r;

	mediump float area = max(max(lookup.a,
								 max(max(lookupL, lookupR),
									 max(lookupT, lookupB))),
							 max(max(lookupTL, lookupTR),
								 max(lookupBL, lookupBR)));



#if kDeblend
	// Deblended Fill

	mediump vec3 source = texel.a > 0.0 ? texel.rgb / texel.a : vec3(0.0);
	mediump float cut = area * uMix * lookup.b;

	mediump vec3 deblended = source - (uOuterColor.rgb * cut);
	mediump vec3 add = uColor.rgb * cut;
	mediump vec4 blended = vec4((deblended + add) * texel.a, texel.a);

	mediump float under = area * (1.0 - uMix) * (1.0 - blended.a);

#if kPreserve
	under *= texel.a;
#endif

	mediump vec4 filled = (uColor * under) + blended;
#else

	// Plain Fill
	mediump float under = area;
#if kPreserve
	under *= texel.a;
#endif

	mediump vec4 filled = mix(texel, uColor, under);
#endif

	filled = clamp(filled, 0.0, min(filled.a, 1.0));


#if kMask
	mediump vec4 mask = texture2D(uTextureMask, vCoordMask);
	filled = mix(texel, filled, mask.r);
#endif

	gl_FragColor = filled;
}