
uniform mediump sampler2D uTextureSource;
uniform mediump sampler2D uTextureStroke;
uniform mediump sampler2D uTextureMask;
#if kTool == kSmudge || kTool == kSmudgePreserve
uniform mediump sampler2D uTextureSmudge;
#endif

uniform mediump vec4 uColor;
uniform mediump float uAlpha;
#if kTool == kSmudge || kTool == kSmudgePreserve
uniform mediump float uCount;
uniform mediump float uWetness;
uniform mediump float uMix;
#endif

varying highp vec2 vCoordStroke;
varying highp vec2 vCoordMask;
#if kTool == kSmudge || kTool == kSmudgePreserve
varying highp vec2 vCoordSmudge;
#endif

#if kTool == kPaint
mediump vec4 paint(const mediump vec4 source, const mediump float stroke);
#endif
#if kTool == kSmudge
mediump vec4 smudge(const mediump vec4 source, const mediump float stroke);
#endif
#if kTool == kErase
mediump vec4 erase(const mediump vec4 source, const mediump float stroke);
#endif
#if kTool == kPaintPreserve
mediump vec4 paintPreserve(const mediump vec4 source, const mediump float stroke);
#endif
#if kTool == kSmudgePreserve
mediump vec4 smudgePreserve(const mediump vec4 source, const mediump float stroke);
#endif


#define MINIMUM_ACCEPTABLE_ALPHA 0.00390625


void main() {
	mediump vec4 source = texture2D(uTextureSource, vCoordStroke);
	mediump float stroke = texture2D(uTextureStroke, vCoordStroke).r * texture2D(uTextureMask, vCoordMask).r * uAlpha;
	
#if kTool == kPaint
	source = paint(source, stroke);
#endif
#if kTool == kSmudge
	source = smudge(source, stroke);
#endif
#if kTool == kErase
	source = erase(source, stroke);
#endif
#if kTool == kPaintPreserve
	source = paintPreserve(source, stroke);
#endif
#if kTool == kSmudgePreserve
	source = smudgePreserve(source, stroke);
#endif
	
	gl_FragColor = clamp(source, 0.0, source.a);
}


#if kTool == kPaint
mediump vec4 paint(const mediump vec4 source, const mediump float stroke)
{
	return mix(source, uColor, stroke);
}
#endif

#if kTool == kSmudge
mediump vec4 smudge(const mediump vec4 source, const mediump float stroke)
{
	mediump float amount = (1.0 - (source.a * uWetness)) * stroke;
	
	highp vec4 floatSmudge = texture2D(uTextureSmudge, vCoordSmudge);
	highp ivec4 intSmudge = ivec4(floatSmudge * 65536.0);
	highp vec4 smudge = vec4(intSmudge / int(uCount)) / 65535.0;
	
	smudge = clamp(smudge, 0.0, min(smudge.a, 1.0));

	return mix(mix(source, smudge, amount), uColor, uMix * stroke);
}
#endif

#if kTool == kErase
mediump vec4 erase(const mediump vec4 source, const mediump float stroke)
{
	return mix(source, vec4(0.0), stroke);
}
#endif

#if kTool == kPaintPreserve
mediump vec4 paintPreserve(const mediump vec4 source, const mediump float stroke)
{
	return vec4(mix(source.rgb, uColor.rgb * source.a, stroke), source.a);
}
#endif

#if kTool == kSmudgePreserve
mediump vec4 smudgePreserve(const mediump vec4 source, const mediump float stroke)
{
	highp vec4 floatSmudge = texture2D(uTextureSmudge, vCoordSmudge);
	highp ivec4 intSmudge = ivec4(floatSmudge * 65536.0);
	highp vec4 smudge = vec4(intSmudge / int(uCount)) / 65535.0;

	smudge = clamp(smudge, 0.0, min(smudge.a, 1.0));

	if (smudge.a < MINIMUM_ACCEPTABLE_ALPHA || source.a < MINIMUM_ACCEPTABLE_ALPHA)
	{
		return source;
	}
	else
	{
		mediump float amount = (1.0 - (source.a * uWetness)) * stroke;
		return mix(mix(source, vec4(smudge.rgb / smudge.a, 1.0) * source.a, amount), uColor * source.a, uMix * stroke);
	}
}
#endif
