
uniform mediump float uThreshold;
uniform mediump vec4 uColor;
uniform mediump sampler2D uTextureSource;

varying highp vec2 vCoord;

#if kMask
uniform mediump sampler2D uTextureMask;
varying highp vec2 vCoordMask;
#endif


void main()
{
	mediump vec4 texel = texture2D(uTextureSource, vCoord);
	mediump vec4 amountV4 = abs(texel - uColor);

	mediump vec3 source = texel.a > 0.0 ? texel.rgb / texel.a : vec3(0.0);
	mediump vec3 color = uColor.a > 0.0 ? uColor.rgb / uColor.a : vec3(0.0);
	
	mediump vec3 alphaV3 = vec3(notEqual(vec3(1.0), source)) * (
																vec3(lessThan(color, source)) * clamp((source - color) / (1.0 - color), 0.0, 1.0) +
																vec3(greaterThan(color, source)) * clamp((color - source) / color, 0.0, 1.0));

	mediump float amount = max(amountV4.r, max(amountV4.g, max(amountV4.b, amountV4.a)));
	mediump float alpha = max(alphaV3.r, max(alphaV3.g, alphaV3.b));

#if kMask
	amount = max(amount, 1.0 - texture2D(uTextureMask, vCoordMask).r);
#endif

	gl_FragColor = vec4(amount <= uThreshold, amount / uThreshold, 1.0 - alpha, 0.0);
}